#include <Arduino.h>

#define PWMRANGE 255

class LED{
	unsigned int _pin;
	bool _on;
	unsigned long _fadeDuration, _fadeStart, _fadeEnd;

	unsigned long getRemainingFadeTime(){
		unsigned long now = millis();
		return _fadeEnd >= now ? constrain(_fadeEnd - now, 0, _fadeDuration) : 0;
	}

	float getMultiplier() {
		float value = getRemainingFadeTime() / (float)_fadeDuration;
		return _on ? value : 1.0f - value;
	}

	void resetTimers(){
		_fadeStart = millis();
		_fadeEnd = (_fadeStart + _fadeDuration) - getRemainingFadeTime();
	}

public:
	LED(unsigned int pin, unsigned long fadeDuration){
		_pin = pin;
		pinMode(pin, OUTPUT);
		_fadeDuration = fadeDuration;
	}

	void on(){
		resetTimers();
		_on = true;
	}

	void off(){
		resetTimers();
		_on = false;
	}

	void loop(){
		float multiplier = getMultiplier();
		analogWrite(_pin, PWMRANGE * multiplier);
	}
};